extends CharacterBody3D


const SPEED = 50.0
const JUMP_VELOCITY = 4.5

# Get the gravity from the project settings to be synced with RigidBody nodes.
var gravity = ProjectSettings.get_setting("physics/3d/default_gravity")


func _ready():
	Input.mouse_mode = Input.MOUSE_MODE_CAPTURED


func _physics_process(delta):
	# Add the gravity.
	if not is_on_floor():
		velocity.y -= gravity * delta

	# Handle Jump.
	if Input.is_action_just_pressed("jump") and is_on_floor():
		velocity.y = JUMP_VELOCITY

	# Get the input direction and handle the movement/deceleration.
	# As good practice, you should replace UI actions with custom gameplay actions.
	var input_dir = Input.get_vector("move_left", "move_right", "move_up", "move_down")
	var direction = (
		transform.basis *
		$Camera/CameraBasis.transform.basis *
		Vector3(input_dir.x, 0, input_dir.y)
		).normalized()
	if direction:
		velocity.x = direction.x * SPEED
		velocity.z = direction.z * SPEED
	else:
		velocity.x = move_toward(velocity.x, 0, SPEED)
		velocity.z = move_toward(velocity.z, 0, SPEED)

	# Shooting the ball
	if Input.is_action_just_pressed("shoot"):
		$Camera/Shooter.shoot($Camera/Shooter.global_position, camera_direction, 10)

	_get_camera_direction()
	move_and_slide()


func _input(event):
	if event is InputEventMouseMotion:
		$Camera.rotation.y -= event.relative.x * (1.0 / 1000)
		$Camera.rotation.x -= event.relative.y * (1.0 / 1000)
		$Camera.rotation.x = clamp($Camera.rotation.x, deg_to_rad(-90), deg_to_rad(90))


var camera_direction : Vector3
func _get_camera_direction():
	camera_direction = $Camera.global_position.direction_to($Camera/CameraDirection.global_position)
